import zmq
from zmq.eventloop.zmqstream import ZMQStream

from tfw.networking.serialization import serialize_all
from tfw.config import tfwenv
from tfw.config.logs import logging
log = logging.getLogger(__name__)
from tfw.networking.zmq_connector_base import ZMQConnectorBase


class EventHandlerDownlinkConnector(ZMQConnectorBase):
    def __init__(self, zmq_context=None):
        super(EventHandlerDownlinkConnector, self).__init__(zmq_context)
        self._zmq_pull_socket = self._zmq_context.socket(zmq.PULL)
        self._zmq_pull_stream = ZMQStream(self._zmq_pull_socket)
        address = 'tcp://*:{}'.format(tfwenv.RECEIVER_PORT)
        self._zmq_pull_socket.bind(address)
        log.debug('Pull socket bound to {}'.format(address))


class EventHandlerUplinkConnector(ZMQConnectorBase):
    def __init__(self, zmq_context=None):
        super(EventHandlerUplinkConnector, self).__init__(zmq_context)
        self._zmq_pub_socket = self._zmq_context.socket(zmq.PUB)
        address = 'tcp://*:{}'.format(tfwenv.PUBLISHER_PORT)
        self._zmq_pub_socket.bind(address)
        log.debug('Pub socket bound to {}'.format(address))


class EventHandlerConnector(EventHandlerDownlinkConnector, EventHandlerUplinkConnector):
    def __init__(self, zmq_context=None):
        super(EventHandlerConnector, self).__init__(zmq_context)

    def register_callback(self, callback):
        self._zmq_pull_stream.on_recv(callback)

    def send_message(self, message: dict, anchor: str = None):
        if not anchor:
            anchor = message['anchor']
        self._zmq_pub_socket.send_multipart(serialize_all(anchor, message))
